@@ -413,127 +413,6 @@ SUBROUTINE PWEFV4(NT,NR,T, R1,R2,R3,R4,TCHANGE,EPS,FX)
    RETURN
 END SUBROUTINE PWEFV4
 
-SUBROUTINE PWEDILDPWU(NT,NR,NU,T,TAUR,U,UT,R1,R2,R3,RC,TCHANGE,EPS,DU,DUPRIME)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'pwedildpwu_' ::PWEDILPWU
-   IMPLICIT NONE
-   INTEGER,INTENT(IN)::NT,NR,NU
-   REAL(8),INTENT(IN)::T(NT),TAUR,U(NU),UT(NU),R1(NR),R2(NR),R3(NR),RC(NR),TCHANGE(NR),EPS
-   REAL(8),INTENT(OUT)::DU(NT),DUPRIME(NT)
-
-   INTEGER::I,N1
-   REAL(8)::UTMINUS(NU),ATEMP,XT(NU+1)
-   REAL(8),ALLOCATABLE::TEMP(:),TM(:,:),FX4(:,:),FX2(:,:),A(:),A0(:),A1(:)
-
-   UTMINUS=0.0
-   IF (NU>1) THEN
-      UTMINUS(2:NU)=UT(1:(NU-1))
-   END IF
-   ATEMP=SUM(U*(UT-UTMINUS))
-   DU=0.0;DUPRIME=0.0
-   N1=NU+1
-   IF (DABS(ATEMP-1.0)<=0.00001) THEN
-      XT(1)=0.0
-      XT(2:N1)=UT
-      ALLOCATE(TEMP(N1),TM(N1,2),FX4(N1,3),FX2(N1,3),A(N1),A0(N1),A1(N1))
-      TM(:,1)=0.0
-      DO I=1,NT,1
-         TM(:,2)=T(I)-XT
-         TEMP=MAXVAL(TM,DIM=2)
-         CALL PWEFV4(N1,NR,TEMP,R2,R2+RC,R3,R1+R3-R2,TCHANGE,EPS,FX4)
-         CALL PWEFV2(N1,NR,TEMP,R1,R1+R3+RC,TCHANGE,EPS,FX2)
-         A=FX4(:,1)+FX2(:,1);A0=TEMP*A;A1=FX4(:,2)+FX2(:,2)
-         DU(I)=SUM(U*(A0(1:NU)-A0(2:N1)))-SUM(U*(A1(1:NU)-A1(2:N1)))
-         DUPRIME(I)=SUM(U*(A(1:NU)-A(2:N1)))
-      END DO
-      DEALLOCATE(TEMP,TM,FX4,FX2,A,A0,A1)
-   END IF
-   RETURN
-END SUBROUTINE PWEDILDPWU
-
-
-SUBROUTINE PWEDILDPWUFORVAR(TFIX,NT,NR,NU,T,TAUR,U,UT,R1,R2,R3,RC,TCHANGE,EPS,F0,F1)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'pwedildpwuforvar_' ::PWEDILPWUFORVAR
-   IMPLICIT NONE
-   INTEGER,INTENT(IN)::NT,NR,NU
-   REAL(8),INTENT(IN)::TFIX,T(NT),TAUR,U(NU),UT(NU),R1(NR),R2(NR),R3(NR),RC(NR),TCHANGE(NR),EPS
-   REAL(8),INTENT(OUT)::F0(NT),F1(NT)
-
-   INTEGER::I,N1,NTEMP
-   REAL(8)::UTMINUS(NU),ATEMP,XT(NU+1),TXMINUS(NU),TX(NU)
-   REAL(8),ALLOCATABLE::TEMP(:),TEMP1(:,:),TEMP0(:,:),FX4(:,:),FX2(:,:),A0(:),A1(:),A2(:),A0T(:),A1T(:),A2T(:),A0X(:),A1X(:),A2X(:)
-
-   UTMINUS=0.0
-   IF (NU>1) THEN
-      UTMINUS(2:NU)=UT(1:(NU-1))
-   END IF
-   TXMINUS=TFIX-UTMINUS
-   TX=TFIX-UT
-   ATEMP=SUM(U*(UT-UTMINUS))
-   F0=0.0;F1=0.0
-   N1=NU+1
-   IF (DABS(ATEMP-1.0)<=0.00001) THEN
-      XT(1)=0.0
-      XT(2:N1)=UT
-      NTEMP=N1+NT
-      ALLOCATE(TEMP(NTEMP),FX4(NTEMP,3),FX2(NTEMP,3),A0(NTEMP),A1(NTEMP),A2(NTEMP))
-      ALLOCATE(A0T(NT),A1T(NT),A2T(NT),A0X(N1),A1X(N1),A2X(N1))
-      ALLOCATE(TEMP1(NU,4),TEMP0(NU,4))
-      TEMP(1:N1)=TFIX-XT
-      TEMP((N1+1):NTEMP)=T
-      CALL PWEFV4(NTEMP,NR,TEMP,R2,R2+RC,R3,R1+R3-R2,TCHANGE,EPS,FX4)
-      CALL PWEFV2(NTEMP,NR,TEMP,R1,R1+R3+RC,TCHANGE,EPS,FX2)
-      A0=FX4(:,1)+FX2(:,1);A1=FX4(:,2)+FX2(:,2);A2=FX4(:,3)+FX2(:,3)
-      A0T=A0((N1+1):NTEMP);A1T=A1((N1+1):NTEMP);A2T=A2((N1+1):NTEMP)
-      A0X=A0(1:N1);A1X=A1(1:N1);A2X=A2(1:N1)
-
-      DO I=1,NT,1
-         TEMP1(:,1)=(TXMINUS*(A1T(I)-A1X(2:N1))-A2T(I)+A2X(2:N1))*U
-         TEMP1(:,2)=(TXMINUS*(A1X(1:NU)-A1X(2:N1))-A2X(1:NU)+A2X(2:N1))*U
-         TEMP1(:,3)=A1X(2:N1)*U*(UT-UTMINUS)
-         TEMP1(:,4)=A1T(I)*U*(UT-UTMINUS)
-
-         TEMP0(:,1)=(TXMINUS*(A0T(I)-A0X(2:N1))-A1T(I)+A1X(2:N1))*U
-         TEMP0(:,2)=(TXMINUS*(A0X(1:NU)-A0X(2:N1))-A1X(1:NU)+A1X(2:N1))*U
-         TEMP0(:,3)=A0X(2:N1)*U*(UT-UTMINUS)
-         TEMP0(:,4)=A0T(I)*U*(UT-UTMINUS)
-
-         F1(I)=SUM(TEMP1(:,1),TXMINUS>T(I) .AND. T(I)>=TX .AND. TX>0.0)
-         F1(I)=F1(I)+SUM(TEMP1(:,2),T(I)>=TXMINUS)
-         F1(I)=F1(I)+SUM(TEMP1(:,3),T(I)>TX)
-         F1(I)=F1(I)+SUM(TEMP1(:,4),TX>=T(I))
-
-         F0(I)=SUM(TEMP0(:,1),TXMINUS>T(I) .AND. T(I)>=TX .AND. TX>0.0)
-         F0(I)=F0(I)+SUM(TEMP0(:,2),T(I)>=TXMINUS)
-         F0(I)=F0(I)+SUM(TEMP0(:,3),T(I)>TX)
-         F0(I)=F0(I)+SUM(TEMP0(:,4),TX>=T(I))
-      END DO
-
-      DEALLOCATE(TEMP,FX4,FX2,A0,A1,A2)
-      DEALLOCATE(A0T,A1T,A2T,A0X,A1X,A2X,TEMP1,TEMP0)
-   END IF
-   RETURN
-END SUBROUTINE PWEDILDPWUFORVAR
-
-SUBROUTINE LDYN(N,Y,YT)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ldyn_' ::LDYN
-   IMPLICIT NONE
-   INTEGER,INTENT(IN)::N
-   REAL(8),INTENT(IN)::Y(N)
-   REAL(8),INTENT(OUT)::YT(N)
-
-   INTEGER::I,J
-   REAL(8),ALLOCATABLE,DIMENSION(:)::T
-
-   YT=0.0
-   DO I=1,N,1
-      ALLOCATE(T(I))
-      T=(/(REAL(J,8),J=1,I)/)
-      YT(1:I)=YT(1:I)+T
-      DEALLOCATE(T)
-   END DO
-   RETURN
-END SUBROUTINE LDYN
-
 SUBROUTINE PWEFV4TYPE2(NT,NR,TIN,R1,R2,R3,R4,TCHANGE,EPS,FX)
 !DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'pwefv4type2_' ::PWEFV4TYPE2
    IMPLICIT NONE
@@ -979,3 +858,127 @@ SUBROUTINE PWEFV4TYPE2(NT,NR,TIN,R1,R2,R3,R4,TCHANGE,EPS,FX)
    END IF
    RETURN
 END SUBROUTINE PWEFV4TYPE2
+
+SUBROUTINE FOURHR(NT,NR,TIN,R1,R2,R3,R4,TCHANGE,EPS,FX)
+!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'fourhr_' ::FOURHR
+   IMPLICIT NONE
+   INTEGER,INTENT(IN)::NT,NR
+   REAL(8),INTENT(IN)::TIN(NT),R1(NR),R2(NR),R3(NR),R4(NR),TCHANGE(NR),EPS
+   REAL(8),INTENT(OUT)::FX(NT)
+
+
+   INTEGER::J,K,N1,POS(NT),INDT(NT),SINDJ1,SINDJ2,SINDJ3,SINDJ4
+   REAL(8)::TMAX,TPLUS(NR),CTEMP,B,A,A1,A2,B12,A12
+   REAL(8),DIMENSION(NR,5)::PR2,PR4
+
+   REAL(8),ALLOCATABLE,DIMENSION(:,:)::TX
+   REAL(8),ALLOCATABLE,DIMENSION(:)::FXX,T,TJ,X2TEMP,X4TEMP,XX,XX1
+   INTEGER,ALLOCATABLE,DIMENSION(:)::INDTS,INDJ,IND,INDS
+
+   FX=0.0;POS=0
+   INDT=(/(J,J=1,NT)/)
+   WHERE (TIN>0.0)
+      POS=1
+   END WHERE
+   N1=COUNT(POS==1)
+
+   IF (N1>0) THEN
+      ALLOCATE(FXX(N1),T(N1),INDTS(N1),IND(N1),INDJ(N1))
+      INDTS=PACK(INDT,POS==1)
+      IND=(/(J,J=1,N1)/)
+      T=PACK(TIN,POS==1)
+      FXX=0.0
+ 
+      TMAX=DMAX1(MAXVAL(T),MAXVAL(TCHANGE))+0.1
+      TPLUS=0.0
+      TPLUS(NR)=TMAX
+      IF (NR>1) THEN
+         TPLUS(1:(NR-1))=TCHANGE(2:NR)
+      END IF
+      
+      CALL PWE(NR,NR,TCHANGE,R2,TCHANGE,PR2)
+      CALL PWE(NR,NR,TCHANGE,R4,TCHANGE,PR4)
+
+      DO J=1,NR,1
+         DO K=1,NR,1
+            B=TPLUS(J)+TPLUS(K)
+            A=TCHANGE(J)+TCHANGE(K)
+            A1=TCHANGE(J)+TPLUS(K)
+            A2=TPLUS(J)+TCHANGE(K)
+            B12=DMAX1(A1,A2)
+            A12=DMIN1(A1,A2)
+            CTEMP=R1(J)*PR2(J,5)*R3(K)*PR4(K,5)
+            WHERE (T<A12 .AND. T>A) 
+                INDJ=1
+            ELSEWHERE (T<A2 .AND. T>=A1)
+                INDJ=2
+            ELSEWHERE (T<A1 .AND. T>=A2)
+                INDJ=3
+            ELSEWHERE (T<B .AND. T>=B12)
+                INDJ=4
+            ELSEWHERE
+                INDJ=0
+            END WHERE
+            SINDJ1=COUNT(INDJ==1)
+            SINDJ2=COUNT(INDJ==2)
+            SINDJ3=COUNT(INDJ==3)
+            SINDJ4=COUNT(INDJ==4)
+            IF (SINDJ1>0) THEN
+               ALLOCATE(INDS(SINDJ1),TJ(SINDJ1),X4TEMP(SINDJ1),XX(SINDJ1),XX1(SINDJ1),TX(SINDJ1,3))
+               INDS=PACK(IND,INDJ==1)
+               TJ=PACK(T,INDJ==1)
+               X4TEMP=TJ-A
+               XX=TJ-A
+               XX1=(R2(J)-R4(K))*XX
+               CALL SPF(SINDJ1,XX1,EPS,TX)
+               FXX(INDS)=FXX(INDS)+CTEMP*DEXP(-R4(K)*X4TEMP)*XX*TX(:,1)
+               DEALLOCATE(INDS,TJ,X4TEMP,XX,XX1,TX)
+            END IF
+
+
+            IF (SINDJ2>0) THEN
+               ALLOCATE(INDS(SINDJ2),TJ(SINDJ2),X2TEMP(SINDJ2),X4TEMP(1),XX(1),XX1(1),TX(1,3))
+               INDS=PACK(IND,INDJ==2)
+               TJ=PACK(T,INDJ==2)
+               X2TEMP=TJ-A1
+               X4TEMP=TPLUS(K)-TCHANGE(K)
+               XX=TPLUS(K)-TCHANGE(K)
+               XX1=(R2(J)-R4(K))*XX
+               CALL SPF(1,XX1,EPS,TX)
+               FXX(INDS)=FXX(INDS)+CTEMP*DEXP(-R2(J)*X2TEMP-R4(K)*X4TEMP(1))*XX(1)*TX(1,1)
+               DEALLOCATE(INDS,TJ,X2TEMP,X4TEMP,XX,XX1,TX)
+            END IF
+
+            IF (SINDJ3>0) THEN
+               ALLOCATE(INDS(SINDJ3),TJ(SINDJ3),X4TEMP(SINDJ3),XX(1),XX1(1),TX(1,3))
+               INDS=PACK(IND,INDJ==3)
+               TJ=PACK(T,INDJ==3)
+               X4TEMP=TJ-A
+               XX=TPLUS(J)-TCHANGE(J)
+               XX1=(R2(J)-R4(K))*XX
+               CALL SPF(1,XX1,EPS,TX)
+               FXX(INDS)=FXX(INDS)+CTEMP*DEXP(-R4(K)*X4TEMP)*XX(1)*TX(1,1)
+               DEALLOCATE(INDS,TJ,X4TEMP,XX,XX1,TX)
+            END IF
+
+
+            IF (SINDJ4>0) THEN
+               ALLOCATE(INDS(SINDJ4),TJ(SINDJ4),X2TEMP(SINDJ4),X4TEMP(1),XX(SINDJ4),XX1(SINDJ4),TX(SINDJ4,3))
+               INDS=PACK(IND,INDJ==4)
+               TJ=PACK(T,INDJ==4)
+               X2TEMP=TJ-A1
+               X4TEMP=TPLUS(K)-TCHANGE(K)
+               XX=B-TJ
+               XX1=(R2(J)-R4(K))*XX
+               CALL SPF(SINDJ4,XX1,EPS,TX)
+               FXX(INDS)=FXX(INDS)+CTEMP*DEXP(-R2(J)*X2TEMP-R4(K)*X4TEMP(1))*XX*TX(:,1)
+               DEALLOCATE(INDS,TJ,X2TEMP,X4TEMP,XX,XX1,TX)
+            END IF
+         END DO
+      END DO
+      FX(INDTS)=FXX
+      DEALLOCATE(FXX,T,INDTS,IND,INDJ)
+   END IF
+   RETURN
+END SUBROUTINE FOURHR
+
